# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.2.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
import exiv2.types

# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._value import *
else:
    from _value import *


import enum

class CharsetIdMeta(enum.EnumMeta):
    def __getattribute__(cls, name):
        obj = super().__getattribute__(name)
        if isinstance(obj, enum.Enum):
            import warnings
            warnings.warn(
                "Use 'CommentValue.CharsetId' instead of 'CharsetId'",
                DeprecationWarning)
        return obj

class DeprecatedCharsetId(enum.IntEnum, metaclass=CharsetIdMeta):
    pass

CharsetId = DeprecatedCharsetId('CharsetId', _enum_list_CharsetId())
CharsetId.__doc__ = "Character set identifiers for the character sets defined by Exif."


import enum

class XmpArrayTypeMeta(enum.EnumMeta):
    def __getattribute__(cls, name):
        obj = super().__getattribute__(name)
        if isinstance(obj, enum.Enum):
            import warnings
            warnings.warn(
                "Use 'XmpValue.XmpArrayType' instead of 'XmpArrayType'",
                DeprecationWarning)
        return obj

class DeprecatedXmpArrayType(enum.IntEnum, metaclass=XmpArrayTypeMeta):
    pass

XmpArrayType = DeprecatedXmpArrayType('XmpArrayType', _enum_list_XmpArrayType())
XmpArrayType.__doc__ = "XMP array types."


import enum

class XmpStructMeta(enum.EnumMeta):
    def __getattribute__(cls, name):
        obj = super().__getattribute__(name)
        if isinstance(obj, enum.Enum):
            import warnings
            warnings.warn(
                "Use 'XmpValue.XmpStruct' instead of 'XmpStruct'",
                DeprecationWarning)
        return obj

class DeprecatedXmpStruct(enum.IntEnum, metaclass=XmpStructMeta):
    pass

XmpStruct = DeprecatedXmpStruct('XmpStruct', _enum_list_XmpStruct())
XmpStruct.__doc__ = "XMP structure indicator."


